@@ -59,19 +59,19 @@ class Connection(Endpoint):
   """
 
   @static
-  def establish(url=None, **options):
+  def establish(url=None, timeout=None, **options):
     """
     Constructs a L{Connection} with the supplied parameters and opens
     it.
     """
     conn = Connection(url, **options)
-    conn.open()
+    conn.open(timeout=timeout)
     return conn
 
   def __init__(self, url=None, **options):
     """
-    Creates a connection. A newly created connection must be connected
-    with the Connection.connect() method before it can be used.
+    Creates a connection. A newly created connection must be opened
+    with the Connection.open() method before it can be used.
 
     @type url: str
     @param url: [ <username> [ / <password> ] @ ] <host> [ : <port> ]
@@ -88,25 +88,24 @@ class Connection(Endpoint):
     @param username: the username for authentication (overriden by url)
     @type password: str
     @param password: the password for authentication (overriden by url)
-
     @type sasl_mechanisms: str
     @param sasl_mechanisms: space separated list of permitted sasl mechanisms
     @type sasl_service: str
-    @param sasl_service: ???
-    @type sasl_min_ssf: ???
-    @param sasl_min_ssf: ???
-    @type sasl_max_ssf: ???
-    @param sasl_max_ssf: ???
+    @param sasl_service: the service name if needed by the SASL mechanism in use
+    @type sasl_min_ssf: int
+    @param sasl_min_ssf: the minimum acceptable security strength factor
+    @type sasl_max_ssf: int
+    @param sasl_max_ssf: the maximum acceptable security strength factor
 
     @type reconnect: bool
     @param reconnect: enable/disable automatic reconnect
     @type reconnect_timeout: float
     @param reconnect_timeout: total time to attempt reconnect
-    @type reconnect_internal_min: float
-    @param reconnect_internal_min: minimum interval between reconnect attempts
-    @type reconnect_internal_max: float
-    @param reconnect_internal_max: maximum interval between reconnect attempts
-    @type reconnect_internal: float
+    @type reconnect_interval_min: float
+    @param reconnect_interval_min: minimum interval between reconnect attempts
+    @type reconnect_interval_max: float
+    @param reconnect_interval_max: maximum interval between reconnect attempts
+    @type reconnect_interval: float
     @param reconnect_interval: set both min and max reconnect intervals
     @type reconnect_limit: int
     @param reconnect_limit: limit the total number of reconnect attempts
@@ -122,6 +121,10 @@ class Connection(Endpoint):
     @param ssl_certfile: file with client's public (eventually priv+pub) key (PEM format)
     @type ssl_trustfile: str
     @param ssl_trustfile: file trusted certificates to validate the server
+    @type ssl_skip_hostname_check: bool
+    @param ssl_skip_hostname_check: disable verification of hostname in
+    certificate. Use with caution - disabling hostname checking leaves you
+    vulnerable to Man-in-the-Middle attacks.
 
     @rtype: Connection
     @return: a disconnected Connection
@@ -170,6 +173,7 @@ class Connection(Endpoint):
     self.ssl_keyfile = options.get("ssl_keyfile", None)
     self.ssl_certfile = options.get("ssl_certfile", None)
     self.ssl_trustfile = options.get("ssl_trustfile", None)
+    self.ssl_skip_hostname_check = options.get("ssl_skip_hostname_check", False)
     self.client_properties = options.get("client_properties", {})
 
     self.options = options
@@ -199,7 +203,13 @@ class Connection(Endpoint):
   def check_error(self):
     if self.error:
       self._condition.gc()
-      raise self.error
+      e = self.error
+      if isinstance(e, ContentError):
+          """ forget the content error. It will be
+          raised this time but won't block future calls
+          """
+          self.error = None
+      raise e
 
   def get_error(self):
     return self.error
@@ -246,14 +256,16 @@ class Connection(Endpoint):
     self.sessions.pop(ssn.name, 0)
 
   @synchronized
-  def open(self):
+  def open(self, timeout=None):
     """
     Opens a connection.
     """
     if self._open:
       raise ConnectionError("already open")
     self._open = True
-    self.attach()
+    if self.reconnect and self.reconnect_timeout > 0:
+        timeout = self.reconnect_timeout
+    self.attach(timeout=timeout)
 
   @synchronized
   def opened(self):
@@ -263,7 +275,7 @@ class Connection(Endpoint):
     return self._open
 
   @synchronized
-  def attach(self):
+  def attach(self, timeout=None):
     """
     Attach to the remote endpoint.
     """
@@ -271,7 +283,9 @@ class Connection(Endpoint):
       self._connected = True
       self._driver.start()
       self._wakeup()
-    self._ewait(lambda: self._transport_connected and not self._unlinked())
+    if not self._ewait(lambda: self._transport_connected and not self._unlinked(), timeout=timeout):
+      self.reconnect = False
+      raise Timeout("Connection attach timed out")
 
   def _unlinked(self):
     return [l
@@ -694,7 +708,7 @@ class Session(Endpoint):
       self._ecwait(lambda: not [m for m in messages if m in self.acked])
 
   @synchronized
-  def commit(self):
+  def commit(self, timeout=None):
     """
     Commit outstanding transactional work. This consists of all
     message sends and receives since the prior commit or rollback.
@@ -703,7 +717,7 @@ class Session(Endpoint):
       raise NontransactionalSession()
     self.committing = True
     self._wakeup()
-    self._ecwait(lambda: not self.committing)
+    self._ecwait(lambda: not self.committing, timeout=timeout)
     if self.aborted:
       raise TransactionAborted()
     assert self.committed
@@ -736,6 +750,7 @@ class Session(Endpoint):
     """
     Close the session.
     """
+    if self.error: return
     self.sync(timeout=timeout)
 
     for link in self.receivers + self.senders:
@@ -845,7 +860,7 @@ class Sender(Endpoint):
     if not self.session.connection._connected or self.session.closing:
       raise Detached()
 
-    self._ecwait(lambda: self.linked)
+    self._ecwait(lambda: self.linked, timeout=timeout)
 
     if isinstance(object, Message):
       message = object
@@ -882,8 +897,14 @@ class Sender(Endpoint):
     if self.synced < mno:
       self.synced = mno
       self._wakeup()
-    if not self._ewait(lambda: self.acked >= mno, timeout=timeout):
-      raise Timeout("sender sync timed out")
+    try:
+      if not self._ewait(lambda: self.acked >= mno, timeout=timeout):
+        raise Timeout("sender sync timed out")
+    except ContentError:
+      # clean bad message so we can continue
+      self.acked = mno
+      self.session.outgoing.pop(0)
+      raise
 
   @synchronized
   def close(self, timeout=None):
@@ -983,7 +1004,7 @@ class Receiver(Endpoint, object):
     """
     Returns the number of acknowledged messages awaiting confirmation.
     """
-    return len([m for m in self.acked if m._receiver is self])
+    return len([m for m in self.session.acked if m._receiver is self])
 
   @synchronized
   def available(self):
